@@ -104,9 +104,9 @@ SEXP
 read_mtp(SEXP fname)
 {
     FILE *f;
-    char buf[MTP_BUF_SIZE], blank[1];
+    char buf[MTP_BUF_SIZE], blank[1], *pres;
     MTB  *mtb, thisRec;
-    int i, j, nMTB = MTB_INITIAL_ENTRIES;
+    int i, j, res, nMTB = MTB_INITIAL_ENTRIES;
 
     PROTECT(fname = asChar(fname));
 #ifdef WIN32 /* force text-mode read */
@@ -120,7 +120,8 @@ read_mtp(SEXP fname)
 	strncmp(buf, "Minitab Portable Worksheet ", 27) != 0)
 	error(_("file '%s' is not in Minitab Portable Worksheet format"),
 	      CHAR(fname));
-    fgets(buf, MTP_BUF_SIZE, f);
+    pres = fgets(buf, MTP_BUF_SIZE, f);
+    if(pres != buf) error(_("file read error"));
     UNPROTECT(1);
 
     mtb = Calloc(nMTB, MTB);
@@ -140,21 +141,25 @@ read_mtp(SEXP fname)
 	case 0:		/* numeric data */
 	    thisRec->dat.ndat = Calloc(thisRec->len, double);
 	    for (j = 0; j < thisRec->len; j++) {
-		fscanf(f, "%lg", thisRec->dat.ndat + j);
+		res = fscanf(f, "%lg", thisRec->dat.ndat + j);
+		if(res == EOF) error(_("file read error"));
 	    }
 	    break;
 	default:
 	    if (thisRec->type == 4) { /* we have a matrix so dtype is number of columns */
 		thisRec->dat.ndat = Calloc(thisRec->len, double);
 		for (j = 0; j < thisRec->len; j++) {
-		    fscanf(f, "%lg", thisRec->dat.ndat + j);
+		    res = fscanf(f, "%lg", thisRec->dat.ndat + j);
+		    if(res == EOF) error(_("file read error"));
 		}
 	    } else {
 		error(_("non-numeric data types are not yet implemented"));
 	    }
 	}
-	fgets(buf, MTP_BUF_SIZE, f); /* clear rest of current line */
-	fgets(buf, MTP_BUF_SIZE, f); /* load next line */
+	pres = fgets(buf, MTP_BUF_SIZE, f); /* clear rest of current line */
+  	if(pres != buf) error(_("file read error"));
+	pres = fgets(buf, MTP_BUF_SIZE, f); /* load next line */
+	/* don't test here, as we test eof at end of loop */
     }
     return MTB2SEXP(mtb, i);
 }
